const User = require("../models/user");
const Staff = require("../models/staff");
const Auth = require("../models/auth");
const bcrypt = require('bcrypt');

const userModel = new User();
const staffModel = new Staff();

const showLogin = async(req, res, next) => {
    res.render("auth/login");
};

const showRegister = async(req, res, next) => {
    res.render("auth/register");
};

const checkAuth = async(req, res, next) => {
    const token = req.session.token;
    if (!token) return res.redirect('/login');
    try {
        const sessionData = await Auth.getSessionData(req.session.token);
        if(!sessionData.isLoggedIn) {
            return res.redirect('/login');
        }
    } catch (error) {
      return res.redirect('/login');
    }
    next();
};

const authRegister = async(req, res, next) => {
    const { phone_number, password, passwordcf } = req.body;

    if(!phone_number || !password || !passwordcf) {
        return res.status(400).json({ error_type: "alert", error: "โปรดกรอกข้อมูลให้ครบถ้วน" });
    }
    const phoneNumberTaken = await userModel.findOneBy("phone_number",phone_number);
    if(phoneNumberTaken != null) return res.status(400).json({ error_type: "email", error: "เบอร์โทรศัพท์ถูกใช้ไปแล้ว" });

    if(password !== passwordcf) {
        return res.status(400).json({ error_type: "password", error: "รหัสผ่านไม่ตรงกัน" });
    }
    const hashedPassword = await bcrypt.hash(password, 10);
    await userModel.create({
        phone_number, password: hashedPassword
    });
    res.redirect("/login?status=success");
};

const authLogin = async(req, res, next) => {
    const { phone_number, password } = req.body;

    if(!phone_number || !password) {
        return res.redirect("/login?status=fail");
    }
    const userResult = await userModel.findOneBy("phone_number",phone_number);
    if(userResult == null) return res.redirect("/login?status=fail");
    const isMatch = await bcrypt.compare(password, userResult.password);
    if (!isMatch) {
        return res.redirect("/login?status=fail");
    }
    const staffResult = await staffModel.findOneBy("user_id",userResult.id);
    await Auth.setJWTSession(req,userResult.id,userResult.phone_number,staffResult ? staffResult.role : "สมาชิก");
    res.redirect("/");
};

const authLogout = async(req, res, next) => {
    req.session = null;
    res.redirect('/login');
};

const adminRoute = async(req, res, next) => {
    const token = req.session.token;
    if (!token) return res.status(403).send("Unauthorized");
    try {
        const sessionData = await Auth.getSessionData(req.session.token);
        if(!sessionData.isLoggedIn) {
            return res.status(403).send("Unauthorized");
        }
        const staffResult = await staffModel.findOneBy("user_id",sessionData.userId);
        if(staffResult == null) {
            return res.status(403).send("Unauthorized");
        }
        else if(staffResult.role !== "Administrator") {
            return res.status(403).send("Unauthorized");
        }
    } catch (error) {
        console.log(error);
      return res.status(403).send("Unauthorized");
    }
    next();
};

const staffRoute = async(req, res, next) => {
    const token = req.session.token;
    if (!token) return res.status(403).send("Unauthorized");
    try {
        const sessionData = await Auth.getSessionData(req.session.token);
        if(!sessionData.isLoggedIn) {
            return res.status(403).send("Unauthorized");
        }
        const staffResult = await staffModel.findOneBy("user_id",sessionData.userId);
        if(staffResult == null) {
            return res.status(403).send("Unauthorized");
        }
    } catch (error) {
        console.log(error);
      return res.status(403).send("Unauthorized");
    }
    next();
};

module.exports = {
    authLogin,
    showLogin,
    showRegister,
    checkAuth,
    authRegister,
    authLogout,
    adminRoute,
    staffRoute,
};