const Auth = require("../models/auth");
const User = require("../models/user");

const bcrypt = require('bcrypt');

const userModel = new User();


const showIndex = async(req, res, next) => {
    const { query } = req.query;
    const authResult = await Auth.getSessionData(req.session.token); 
    var userResult;
    if(!query) {
        userResult = await userModel.findAll() || [ ];
    }
    else userResult = await userModel.searchBy('phone_number',query) || [ ];

    const formattedResults = userResult.map(item => {
        const date = new Date(item.created_at);
        const buddhaYear = date.getFullYear() + 543;
        const formattedDate = `${date.getDate().toString().padStart(2, '0')}/${
            (date.getMonth() + 1).toString().padStart(2, '0')
        }/${buddhaYear} ${date.getHours().toString().padStart(2, '0')}:${
            date.getMinutes().toString().padStart(2, '0')
        }:${date.getSeconds().toString().padStart(2, '0')}`;

        return {
            ...item,
            created_at: formattedDate,
        };
    });

    res.render("restaurant/user/index", { role: authResult.role,userData: formattedResults });
};

const showCreate = async(req, res, next) => {
    const authResult = await Auth.getSessionData(req.session.token); 

    res.render("restaurant/user/create", { role: authResult.role });
};

const showManage = async(req, res, next) => {
    const id = req.params.id;
    const authResult = await Auth.getSessionData(req.session.token); 
    const userResult = await userModel.findOneBy('id',id);
    if(userResult == null) return res.status(404).send("Not Found.");

    res.render("restaurant/user/manage", { role: authResult.role,userData: userResult,isSelf: userResult.id == authResult.userId ? true : false });
};

const createUser = async(req, res, next) => { 
    const { phone_number, password, passwordcf } = req.body;

    if(!phone_number || !password || !passwordcf) {
        return res.status(400).json({ error_type: "alert", error: "โปรดกรอกข้อมูลให้ครบถ้วน" });
    }
    const phoneNumberTaken = await userModel.findOneBy("phone_number",phone_number);
    if(phoneNumberTaken != null) return res.status(400).json({ error_type: "email", error: "เบอร์โทรศัพท์ถูกใช้ไปแล้ว" });

    if(password !== passwordcf) {
        return res.status(400).json({ error_type: "password", error: "รหัสผ่านไม่ตรงกัน" });
    }
    const hashedPassword = await bcrypt.hash(password, 10);
    await userModel.create({
        phone_number, password: hashedPassword
    });
    res.redirect("/user");
};

const updateUser = async(req, res, next) => { 
    const { id, phone_number, password, passwordcf } = req.body;
    const userResult = await userModel.findOneBy('id',id);
    if(userResult == null) return res.status(404).send("Not Found.");

    if(!phone_number) {
        return res.status(400).json({ error_type: "alert", error: "โปรดกรอกข้อมูลให้ครบถ้วน" });
    }

    
    if(password && passwordcf) {
        if(password !== passwordcf) {
            return res.status(400).json({ error_type: "password", error: "รหัสผ่านไม่ตรงกัน" });
        }
        await userModel.updateBy('id',id, { phone_number,password });
    }
    else await userModel.updateBy('id',id, { phone_number });
    res.redirect("/user/manage/" + id);
};

const deleteUser = async(req, res, next) => { 
    const { id } = req.body;

    if(!id) {
        return res.status(400).json({ error_type: "alert", error: "โปรดกรอกข้อมูลให้ครบถ้วน" });
    }
    const authResult = await Auth.getSessionData(req.session.token); 
    if(id == authResult.userId) {
        return res.status(400).send("You cannot delete yourself.");
    }

    await userModel.deleteById(id);

    res.redirect("/user");
};

module.exports = {
    showIndex,
    showCreate,
    showManage,
    createUser,
    updateUser,
    deleteUser
};